import json
import os

import requests

from staxapp.api import Api
from staxapp.auth import ApiTokenAuth
from staxapp.config import Config
from staxapp.contract import StaxContract
from staxapp.exceptions import ApiException, ValidationException


class StaxClient:
    _operation_map = dict()
    _schema = dict()
    _initialized = False

    def __init__(self, classname, lambda_client=None):
        # Stax feature, eg 'quotas', 'workloads'
        self.classname = classname

        if not self._initialized:
            self._map_paths_to_operations()
            StaxContract.set_schema(self._schema)
            if not self._operation_map.get(self.classname):
                raise ValidationException(
                    f"No such class: {self.classname}. Please use one of {list(self._operation_map)}"
                )
            self._initialized = True

        if lambda_client:
            self.lambda_client = lambda_client
            self._admin = True
            self.arn = f"arn:aws:lambda:{Config.api_config['Juma']['controlplaneRegion']}:{Config.api_config['Juma']['masterAccountId']}:function:{self.classname}-admninrpc-{Config.api_config['Juma']['stage']}-{Config.branch()}"
        else:
            Config.auth_class = ApiTokenAuth
            self._admin = False

    @classmethod
    def _load_schema(cls):
        if Config.load_live_schema:
            schema_response = requests.get(Config.schema_url())
            schema_response.raise_for_status()
            cls._schema = schema_response.json()
        else:
            current_dir = os.path.abspath(os.path.dirname(__file__))
            schema_file = f"{current_dir}/data/schema.json"
            with open(schema_file, "r") as f:
                cls._schema = json.load(f)

    @classmethod
    def _map_paths_to_operations(cls):
        cls._load_schema()
        for path_name, path in cls._schema["paths"].items():
            parameters = []
            base_path = ""
            path_parts = path_name.split("/")

            for part in path_parts:
                if "{" in part:
                    parameters.append(part.replace("{", "").replace("}", ""))
                else:
                    base_path = f"{base_path}/{part}"

            for method_type, method in path.items():
                method = path[method_type]
                operation = method.get("operationId", "").split(".")

                if len(operation) != 2:
                    continue

                api_class = operation[0]
                method_name = operation[1]

                if not cls._operation_map.get(api_class):
                    cls._operation_map[api_class] = dict()
                if not cls._operation_map.get(api_class, {}).get(method_name):
                    cls._operation_map[api_class][method_name] = dict()
                    cls._operation_map[api_class][method_name]["path"] = base_path
                    cls._operation_map[api_class][method_name]["method"] = method_type
                    cls._operation_map[api_class][method_name]["parameters"] = []
                cls._operation_map[api_class][method_name]["parameters"].append(
                    parameters
                )

    def __getattr__(self, name):
        self.name = name

        def stax_wrapper(*args, **kwargs):
            method_name = f"{self.classname}.{self.name}"
            method = self._operation_map[self.classname].get(self.name)
            if method is None:
                raise ValidationException(
                    f"No such operation: {self.name} for {self.classname}. Please use one of {list(self._operation_map[self.classname])}"
                )
            payload = {**kwargs}
            parameters = ""
            # All parameters starting with the most dependant
            operation_parameters = self._operation_map[self.classname][self.name].get(
                "parameters", []
            )
            # Sort the operation map parameters
            sorted_operation_parameters = sorted(
                operation_parameters, key=len, reverse=True
            )

            # Check if the any of the parameter schemas match parameters provided
            for parameter_list in sorted_operation_parameters:
                # Get any parameters from the keyword args and remove them from the payload
                if set(parameter_list).issubset(payload.keys()):
                    for parameter in parameter_list:
                        parameters = f"{parameters}/{payload.pop(parameter, None)}"
            if parameters.count("/") < len(sorted_operation_parameters[-1]):
                raise ValidationException(
                    f"Missing one or more parameters: {sorted_operation_parameters[-1]}"
                )

            if method["method"].lower() in ["put", "post"]:
                # We only validate the payload for POST/PUT routes
                StaxContract.validate(payload, method_name)
            ret = getattr(Api, method["method"])(
                f'{method["path"]}{parameters}', payload
            )
            return ret

        return stax_wrapper



import json
import logging
import os
from contextlib import suppress

import requests
from jsonschema import validate as json_validate
from prance import ResolvingParser

from staxapp.config import Config
from staxapp.exceptions import ValidationException

logging.getLogger("openapi_spec_validator.validators").setLevel(logging.WARNING)


class StaxContract:
    _swagger_doc = None
    _resolved_schema = None

    @staticmethod
    def resolve_schema_refs(schema) -> dict:
        """Replaces the $refs within an openapi3.0 schema with the referenced components"""
        parser = ResolvingParser(spec_string=schema, backend="openapi-spec-validator")
        return parser.specification

    @classmethod
    def get_schema(cls):
        return cls._swagger_doc

    @classmethod
    def set_schema(cls, schema):
        cls._swagger_doc = schema
        cls._resolved_schema = cls.resolve_schema_refs(json.dumps(cls._swagger_doc))

    @classmethod
    def validate(cls, data, component):
        """
        Validates a request body against an component in a openapi3.0 template
        """
        if not cls._swagger_doc:
            cls.set_schema(cls.default_swagger_template())

        components = cls._resolved_schema.get("components")
        if components is not None:
            schemas = {**components.get("schemas", {})}
            if component in schemas:
                try:
                    json_validate(instance=data, schema=schemas[component])
                except Exception as err:
                    raise ValidationException(str(err))
            else:
                raise ValidationException(f"SCHEMA: Does not contain {component}")

    @staticmethod
    def default_swagger_template() -> dict:
        # Get the default swagger template from https://api.au1.staxapp.cloud/20190206/public/api-document
        schema_response = requests.get(Config.schema_url()).json()
        template = dict(
            openapi="3.0.0",
            info={
                "title": f"Stax Core API",
                "version": f"{os.getenv('GIT_VERSION')}",
                "description": f"The Stax API is organised around REST, uses resource-oriented URLs, return responses are JSON and uses standard HTTP response codes, authentication and verbs.",
                "termsOfService": "/there_is_no_tos",
                "contact": {"url": "https://stax.io"},
            },
            servers=[{"url": f"https://{Config.hostname}"}],
            paths=dict(),
            components={
                "securitySchemes": {
                    "sigv4": {
                        "type": "apiKey",
                        "name": "Authorization",
                        "in": "header",
                        "x-amazon-apigateway-authtype": "awsSigv4",
                    }
                },
                "schemas": schema_response.get("components").get("schemas"),
                "responses": dict(),
                "requestBodies": dict(),
            },
        )

        return template



#!/usr/local/bin/python3
from datetime import datetime, timezone

import boto3
from aws_requests_auth.aws_auth import AWSRequestsAuth
from botocore import UNSIGNED
from botocore.client import Config as BotoConfig
from botocore.exceptions import ClientError
from warrant import AWSSRP

from staxapp.config import Config as StaxConfig
from staxapp.exceptions import InvalidCredentialsException


class StaxAuth:
    def __init__(self, config_branch):
        config = StaxConfig.api_config

        self.identity_pool = config.get(config_branch).get("identityPoolId")
        self.user_pool = config.get(config_branch).get("userPoolId")
        self.client_id = config.get(config_branch).get("userPoolWebClientId")
        self.aws_region = config.get(config_branch).get("region")

    def requests_auth(self, username, password, **kwargs):
        if username is None:
            raise InvalidCredentialsException(
                "Please provide an Access Key to your config"
            )
        if password is None:
            raise InvalidCredentialsException(
                "Please provide a Secret Key to your config"
            )

        id_token = self.id_token_from_cognito(username, password, **kwargs)
        id_creds = self.sts_from_cognito_identity_pool(id_token, **kwargs)
        auth = self.sigv4_signed_auth_headers(id_creds)

        StaxConfig.expiration = id_creds.get("Credentials").get("Expiration")
        StaxConfig.auth = auth

        return StaxConfig.auth

    def id_token_from_cognito(
        self, username=None, password=None, srp_client=None, **kwargs
    ):
        token = None
        if not srp_client:
            srp_client = boto3.client(
                "cognito-idp",
                region_name=self.aws_region,
                config=BotoConfig(signature_version=UNSIGNED),
            )
        aws = AWSSRP(
            username=username,
            password=password,
            pool_id=self.user_pool,
            client_id=self.client_id,
            client=srp_client,
        )
        try:
            tokens = aws.authenticate_user()
        except ClientError as e:
            if e.response["Error"]["Code"] == "NotAuthorizedException":
                raise InvalidCredentialsException(
                    message=str(e), detail="Please check your Secret Key is correct"
                )
            elif e.response["Error"]["Code"] == "UserNotFoundException":
                raise InvalidCredentialsException(
                    message=str(e),
                    detail="Please check your Access Key, that you have created your Api Token and that you are using the right STAX REGION",
                )
            else:
                raise InvalidCredentialsException(
                    f"Unexpected Client Error. Error details: {e}"
                )
        token = tokens["AuthenticationResult"]["IdToken"]
        return token

    def sts_from_cognito_identity_pool(self, token, cognito_client=None, **kwargs):
        if not cognito_client:
            cognito_client = boto3.client(
                "cognito-identity",
                region_name=self.aws_region,
                config=BotoConfig(signature_version=UNSIGNED),
            )
        try:
            id = cognito_client.get_id(
                IdentityPoolId=self.identity_pool,
                Logins={
                    f"cognito-idp.{self.aws_region}.amazonaws.com/{self.user_pool}": token
                },
            )
            id_creds = cognito_client.get_credentials_for_identity(
                IdentityId=id["IdentityId"],
                Logins={
                    f"cognito-idp.{self.aws_region}.amazonaws.com/{self.user_pool}": token
                },
            )
        except ClientError as e:
            raise InvalidCredentialsException(
                f"Unexpected Client Error. Error details: {e}"
            )
        return id_creds

    def sigv4_signed_auth_headers(self, id_creds):
        auth = AWSRequestsAuth(
            aws_access_key=id_creds.get("Credentials").get("AccessKeyId"),
            aws_secret_access_key=id_creds.get("Credentials").get("SecretKey"),
            aws_token=id_creds.get("Credentials").get("SessionToken"),
            aws_host=f"{StaxConfig.hostname}",
            aws_region=self.aws_region,
            aws_service="execute-api",
        )
        return auth


class RootAuth:
    @staticmethod
    def requests_auth(username, password, **kwargs):
        if StaxConfig.expiration and StaxConfig.expiration > datetime.now(timezone.utc):
            return StaxConfig.auth

        return StaxAuth("JumaAuth").requests_auth(username, password, **kwargs)


class ApiTokenAuth:
    @staticmethod
    def requests_auth(username, password, **kwargs):
        if StaxConfig.expiration and StaxConfig.expiration > datetime.now(timezone.utc):
            return StaxConfig.auth

        return StaxAuth("ApiAuth").requests_auth(username, password, **kwargs)



import logging
import os
import platform as sysinfo

import requests

import staxapp
from staxapp.exceptions import ApiException

logging.getLogger().setLevel(logging.DEBUG)
logging.getLogger("boto3").setLevel(logging.WARNING)
logging.getLogger("botocore").setLevel(logging.WARNING)
logging.getLogger("nose").setLevel(logging.WARNING)
logging.getLogger("urllib3").setLevel(logging.WARNING)


class Config:
    """
    Insert doco here
    """

    STAX_REGION = os.getenv("STAX_REGION", "au1.staxapp.cloud")
    API_VERSION = "20190206"

    api_config = dict()
    access_key = None
    secret_key = None
    auth_class = None
    _initialized = False
    base_url = None
    hostname = f"api.{STAX_REGION}"
    org_id = None
    auth = None
    expiration = None
    load_live_schema = True

    platform = (sysinfo.platform(),)
    python_version = (sysinfo.python_version(),)
    sdk_version = (staxapp.__version__,)

    @classmethod
    def set_config(cls):
        cls.base_url = f"https://{cls.hostname}/{cls.API_VERSION}"
        config_url = f"{cls.api_base_url()}/public/config"
        config_response = requests.get(config_url)
        try:
            config_response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            logging.error(f"{config_response.status_code}: {config_response.json()}")
            raise ApiException(
                str(e), config_response, detail=" Could not load API config."
            )

        cls.api_config = config_response.json()

    @classmethod
    def init(cls, config=None):
        if cls._initialized:
            return

        if not config:
            cls.set_config()

        cls._initialized = True

    @classmethod
    def api_base_url(cls):
        return cls.base_url

    @classmethod
    def branch(cls):
        return os.getenv("STAX_BRANCH", "master")

    @classmethod
    def schema_url(cls):
        return f"{cls.base_url}/public/api-document"

    @classmethod
    def get_auth_class(cls):
        if cls.auth_class is None:
            from staxapp.auth import ApiTokenAuth

            cls.auth_class = ApiTokenAuth
        return cls.auth_class


Config.init()



import json

import requests

from staxapp.config import Config
from staxapp.exceptions import ApiException


class Api:
    _requests_auth = None

    @classmethod
    def _headers(cls, custom_headers) -> dict:
        headers = {
            **custom_headers,
            "User-Agent": json.dumps(
                {
                    "platform": Config.platform,
                    "python_version": Config.python_version,
                    "sdk_version": Config.sdk_version,
                }
            ),
        }
        return headers

    @classmethod
    def _auth(cls, **kwargs):
        if not cls._requests_auth:
            cls._requests_auth = Config.get_auth_class().requests_auth(
                Config.access_key, Config.secret_key, **kwargs
            )
        return cls._requests_auth

    @staticmethod
    def handle_api_response(response):
        try:
            response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            raise ApiException(str(e), response)

    @classmethod
    def get(cls, url_frag, params={}, **kwargs):
        url_frag = url_frag.replace(f"/{Config.API_VERSION}", "")
        url = f"{Config.api_base_url()}/{url_frag.lstrip('/')}"

        response = requests.get(
            url,
            auth=cls._auth(),
            params=params,
            headers=cls._headers(kwargs.get("headers", {})),
            **kwargs,
        )
        cls.handle_api_response(response)
        return response.json()

    @classmethod
    def post(cls, url_frag, payload={}, **kwargs):
        url_frag = url_frag.replace(f"/{Config.API_VERSION}", "")
        url = f"{Config.api_base_url()}/{url_frag.lstrip('/')}"

        response = requests.post(
            url,
            json=payload,
            auth=cls._auth(),
            headers=cls._headers(kwargs.get("headers", {})),
            **kwargs,
        )
        cls.handle_api_response(response)
        return response.json()

    @classmethod
    def put(cls, url_frag, payload={}, **kwargs):
        url_frag = url_frag.replace(f"/{Config.API_VERSION}", "")
        url = f"{Config.api_base_url()}/{url_frag.lstrip('/')}"

        response = requests.put(
            url,
            json=payload,
            auth=cls._auth(),
            headers=cls._headers(kwargs.get("headers", {})),
            **kwargs,
        )
        cls.handle_api_response(response)
        return response.json()

    @classmethod
    def delete(cls, url_frag, params={}, **kwargs):
        url_frag = url_frag.replace(f"/{Config.API_VERSION}", "")
        url = f"{Config.api_base_url()}/{url_frag.lstrip('/')}"

        response = requests.delete(
            url,
            auth=cls._auth(),
            params=params,
            headers=cls._headers(kwargs.get("headers", {})),
            **kwargs,
        )
        cls.handle_api_response(response)
        return response.json()



from setuptools import setup, find_packages
import os
import re

requires = [
    "boto3",
    "prance",
    "requests",
    "aws_requests_auth",
    "warrant",
    "pycrypto",
    "pyjwt",
    "openapi-spec-validator",
]


ROOT = os.path.dirname(__file__)
VERSION_RE = re.compile(r"""__version__ = ['"]([0-9.]+)['"]""")


def get_version():
    init = open(os.path.join(ROOT, "staxapp", "__init__.py")).read()
    return VERSION_RE.search(init).group(1)


setup(
    name="staxapp",
    version=get_version(),
    description="Stax Python SDK",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://stax.io",
    author="Stax",
    author_email="support@stax.io",
    license="Apache License 2.0",
    packages=find_packages(exclude=["docs", "tests*"]),
    install_requires=requires,
    include_package_data=True,
    package_data={"staxapp": ["data/*.json"]},
    zip_safe=False,
    python_requires=">=3.6",
    project_urls={"GitHub": "https://github.com/stax-labs/lib-stax-python-sdk"},
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
)
